@@ -15,7 +15,7 @@
 #include "util.h"
 
 #define ACCEPT_TIMEOUT_SECS 1
-#define SHORT_TIMEOUT_SECS 2
+#define READ_TIMEOUT_SECS 30 
 
 #define MAX_BUFSIZE 1024 * 8
 #define INPUT_BUF_SIZE 1024 * 8
@@ -41,6 +41,7 @@ static int is_keep_alive = 0; //keep alive support
 static int keep_alive_timeout = 5;
 
 int max_content_length = 1024 * 1024 * 16; //max_content_length
+int client_body_buffer_size = 1024 * 500;  //client_body_buffer_size
 
 static char *unix_sock_name = NULL;
 
@@ -480,6 +481,9 @@ w_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 #ifdef DEBUG
     printf("call w_callback \n");
 #endif
+    if(client->environ){
+        current_client = PyDict_GetItem(client->environ, client_key);
+    }
     if ((events & PICOEV_TIMEOUT) != 0) {
 
 #ifdef DEBUG
@@ -732,7 +736,7 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
         char buf[INPUT_BUF_SIZE];
         ssize_t r;
         if(!cli->keep_alive){
-            picoev_set_timeout(loop, cli->fd, SHORT_TIMEOUT_SECS);
+            picoev_set_timeout(loop, cli->fd, READ_TIMEOUT_SECS);
         }
         r = read(cli->fd, buf, sizeof(buf));
         switch (r) {
@@ -1318,6 +1322,26 @@ meinheld_get_max_content_length(PyObject *self, PyObject *args)
     return Py_BuildValue("i", max_content_length);
 }
 
+PyObject *
+meinheld_set_client_body_buffer_size(PyObject *self, PyObject *args)
+{
+    int temp;
+    if (!PyArg_ParseTuple(args, "i", &temp))
+        return NULL;
+    if(temp <= 0){
+        PyErr_SetString(PyExc_ValueError, "client_body_buffer_size value out of range ");
+        return NULL;
+    }
+    client_body_buffer_size = temp;
+    Py_RETURN_NONE;
+}
+
+PyObject *
+meinheld_get_client_body_buffer_size(PyObject *self, PyObject *args)
+{
+    return Py_BuildValue("i", client_body_buffer_size);
+}
+
 PyObject *
 meinheld_set_listen_socket(PyObject *self, PyObject *args)
 {
@@ -1579,7 +1603,21 @@ meinheld_trampoline(PyObject *self, PyObject *args, PyObject *kwargs)
 
 }
 
-
+PyObject *
+meinheld_get_ident(PyObject *self, PyObject *args)
+{
+    if(current_client){
+        ClientObject *pyclient = (ClientObject *)current_client;
+        if(pyclient->greenlet){
+#ifdef DEBUG
+            printf("get thread ident %p\n", pyclient->greenlet);
+#endif
+            Py_INCREF(pyclient->greenlet);
+            return pyclient->greenlet;
+        }
+    }
+    Py_RETURN_NONE;
+}
 
 
 static PyMethodDef WsMethods[] = {
@@ -1593,6 +1631,9 @@ static PyMethodDef WsMethods[] = {
     {"set_max_content_length", meinheld_set_max_content_length, METH_VARARGS, "set max_content_length"},
     {"get_max_content_length", meinheld_get_max_content_length, METH_VARARGS, "return max_content_length"},
     
+    {"set_client_body_buffer_size", meinheld_set_client_body_buffer_size, METH_VARARGS, "set client_body_buffer_size"},
+    {"get_client_body_buffer_size", meinheld_get_client_body_buffer_size, METH_VARARGS, "return client_body_buffer_size"},
+
     {"set_backlog", meinheld_set_backlog, METH_VARARGS, "set backlog size"},
     {"get_backlog", meinheld_get_backlog, METH_VARARGS, "return backlog size"},
 
@@ -1612,6 +1653,7 @@ static PyMethodDef WsMethods[] = {
     // io
     {"cancel_wait", meinheld_cancel_wait, METH_VARARGS, "cancel wait"},
     {"trampoline", (PyCFunction)meinheld_trampoline, METH_VARARGS | METH_KEYWORDS, "trampoline"},
+    {"get_ident", meinheld_get_ident, METH_VARARGS, "return thread ident "},
 
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
